/*******************************************************************************
 * Copyright (c) 2012 itemis AG (http://www.itemis.eu) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.xpect.runner;

import java.lang.reflect.InvocationTargetException;

import org.junit.runner.Description;
import org.xpect.XjmMethod;
import org.xpect.XjmTestMethod;
import org.xpect.setup.ThisTestClass;
import org.xpect.setup.ThisTestObject;
import org.xpect.state.Configuration;
import org.xpect.state.StateContainer;

import com.google.common.base.Preconditions;

/**
 * @author Moritz Eysholdt - Initial contribution and API
 */
public class TestRunner extends AbstractTestRunner {

	private XjmTestMethod method;

	private final StateContainer state;

	public TestRunner(XpectFileRunner uriRunner, XjmTestMethod method) {
		super(uriRunner);
		Preconditions.checkNotNull(method);
		this.method = method;
		this.state = createState(createConfiguration());
	}

	protected Configuration createConfiguration() {
		Configuration config = new Configuration();
		config.addValue(ThisTestClass.class, Class.class, this.method.getTest().getJavaClass());
		config.addDefaultValue(XjmTestMethod.class, this.method);
		config.addDefaultValue(XjmMethod.class, this.method);
		return config;
	}

	public Description createDescription() {
		Class<?> javaClass = getUriRunner().getRunner().getTestClass().getJavaClass();
		return Description.createTestDescription(javaClass, getTitle());
	}

	public XjmTestMethod getMethod() {
		return method;
	}

	@Override
	public StateContainer getState() {
		return state;
	}

	@Override
	protected String getTitle() {
		return method.getName();
	}

	@Override
	protected void runInternal() throws Throwable {
		Object test = state.get(Object.class, ThisTestObject.class).get();
		// Object test = method.getTest().getJavaClass().newInstance();
		// ctx.setMethod(method);
		// ctx.setTestInstance(test);
		try {
			// if (setup != null)
			// ctx.setUserTestCtx(setup.beforeTest(ctx, ctx.getUserFileCtx()));
			method.getJavaMethod().invoke(test);
		} catch (InvocationTargetException e) {
			throw e.getCause();
		}
		// finally {
		// if (setup != null)
		// setup.afterTest(ctx, ctx.getUserTestCtx());
		// }
	}

}
